-- addon notifies the player when somebody enters or leaves the sector

spuck = spuck or {}

local MODE_NONE = 0
local MODE_PLAYER = 1
local MODE_ALL = 2

local mode_patterns = {}
mode_patterns[MODE_PLAYER] = "^[^*]"
mode_patterns[MODE_ALL] = ""


local SOUND_OFF = 0
local SOUND_ENTERED = 1
local SOUND_LEFT = 2
local SOUND_BOTH = 3


local enterleave = {}
enterleave.newchars = {}
enterleave.knownchars = {}

-- get settings
local notify_mode = gkini.ReadInt("*enterleave", "notification_mode", MODE_PLAYER)
local notify_mode_pattern = mode_patterns[notify_mode]

local sound = gkini.ReadInt("*enterleave", "sound", SOUND_ENTERED)
local patterns = unspickle(gkini.ReadString("*enterleave", "patterns", ""))


function enterleave:EnterNotify(name)
	if sound == SOUND_ENTERED or sound == SOUND_BOTH then gksound.GKPlaySound("prox.warning") end
	print("\127ffffff" .. name .." has entered the sector ") 
end

function enterleave:LeaveNotify(name)
	if sound == SOUND_LEFT or sound == SOUND_BOTH then gksound.GKPlaySound("prox.warning") end
	print("\127ffffff" .. name .." has left the sector") 
end

function enterleave:OnEvent(event, data)
	local function isname(name, charid)
		return not (name == "(reading transponder "..charid..")")
	end

	local function matches_pattern(name)
		-- check if mode is valid and return true if it matches
		if notify_mode_pattern and name:match(notify_mode_pattern) then
			return true
		end
	end

	local function matches_custom_pattern(name)
		for _,pattern in ipairs(patterns) do
			if name:match(pattern) then return true end
		end
	end


	local playername = GetPlayerName(data) or "none"
		
	if event == "PLAYER_ENTERED_SECTOR" then
		-- don't do anything if the char is the player or sector
		if data == GetCharacterID() or data == 0 then
			return
		end

		if self.knownchars[data] then
			return
		end
		self.knownchars[data] = true


		if isname(playername, data) then
			if matches_custom_pattern(playername) or matches_pattern(playername) then
				self:EnterNotify(playername)
			end
		else
			-- if this is not a name put it into newplayer table and wait for updates
			self.newchars[data] = true
		end
	end

	if event == "PLAYER_LEFT_SECTOR" then
		-- do nothing if it's the sector (not sure if this actually happens)
		if data == 0 then return end

		-- if the player still has a node id he didn't actually leave
		-- caused by spurious events when players undock and storm sector
		if GetPlayerNodeID(data) then
			return
		end
		self.knownchars[data] = nil

		if isname(playername, data) then
			-- clear temporary char list when the player leaves sector. In case some names got lost
			if data == GetCharacterID() then 
				self.newchars = {}
				self.knownchars = {}
			-- pattern matching
			elseif matches_custom_pattern(playername) or matches_pattern(playername)  then
				self:LeaveNotify(playername)
			end
		end

		self.newchars[data] = nil
	end
	
	if event == "UPDATE_CHARINFO" then
		if self.newchars[data] then
			-- still don't have name so leave it in the table
			if isname(playername, data) then
				self.newchars[data] = nil

				if matches_custom_pattern(playername) or matches_pattern(playername) then
					self:EnterNotify(playername)
				end
			end
		end
	end

	if event == "SECTOR_LOADED" then
		for i in pairs(self.newchars) do
			playername = GetPlayerName(i) or "none"
			if isname(playername, i) then
				self.newchars[i] = nil

				if matches_custom_pattern(playername) or matches_pattern(playername) then
					self:EnterNotify(playername)
				end
			end
		end
	end
end


RegisterEvent(enterleave, "PLAYER_ENTERED_SECTOR")
RegisterEvent(enterleave, "PLAYER_LEFT_SECTOR")
RegisterEvent(enterleave, "UPDATE_CHARINFO")
RegisterEvent(enterleave, "SECTOR_LOADED")


local function notifymode_func(_, args)
	if args and args[1] then 
		local mode = tonumber(args[1]) or notify_mode
		gkini.WriteInt("*enterleave", "notification_mode", mode)
		notify_mode = mode
		notify_mode_pattern = mode_patterns[mode]
	else
		print("usage: /enterleave_notify_mode <number>\n 0 = no notification\n 1 = only players\n 2 = everything")
	end 
end
RegisterUserCommand("enterleave_notify_mode", notifymode_func)

local function pattern_func(_, args)
	if args and args[1] then 
		gkini.WriteString("*enterleave", "patterns", spickle(args))
		patterns = args
	else
		gkini.WriteString("*enterleave", "patterns", "")
		patterns = {}
	end
end
RegisterUserCommand("enterleave_pattern", pattern_func)

local function sound_func(_, args)
	if args and args[1] then 
		local mode = tonumber(args[1]) or sound
		gkini.WriteInt("*enterleave", "sound", mode)
		sound = mode
	else
		print("usage: /enterleave_sound <number>\n 0 = off\n 1 = enter\n 3 = leave\n 4 = both")
	end 
end
RegisterUserCommand("enterleave_sound", sound_func)


spuck.enterleave = enterleave
