--[[

 TA (trade assistant) is a local caching database with query/compare for station commodity's  by Mad Miner Moda.

 2 key critical parts to this.

1) dock,
	add station to list if not there already.
	get price for all station items for sale
	get sell prices for all items in your inventory, ships, items, etdc..
	
2) query/compare
	show 2 stations.
	pick buy/sell stations.
	show profit/loss between stations A and B.

:notes:

negative profit does not show up.
profit marked with a ''p' is unknown volume so assuming 1 cu.
cu or mass marked with a ?? is unknown.

for the buy/sell stations to work you MUST have items from teh 'buy' station at the 'sell' station for it to register.\
trade tab will show all stations that buy or sell that you have visited.

1.5 	saves data
1.9 	bug fixes, color format change, event processing changes
1.11  bug with new items in unknown classes.
1.12	bug with event system_best_price, disabled all best price list for now.
]]

local alpha, selalpha = ListColors.Alpha, ListColors.SelectedAlpha
local even, odd = ListColors[0], ListColors[1]

local bg = {
        [0] = even.." "..alpha,
        [1] = odd.." "..alpha,
}
local bg_numbers = ListColors.Numbers
local sorted = {}
local startup = true

TA = TA or {} -- all non-ui stuff here
TA.stations = {} -- list of stations and related info here.  holds faction, location, etc.. 113 stations
TA.goods = {} -- trade goods table. this is where the magic happens. 405 trade items
TA.version = 'Trade Assistant v1.12\t\t\t(a TGFT Production)'
TA.ui = TA.ui or {} -- all ui stuff here.
TA.ui.Show = {} -- this will show the ui

TA.ui.color = { -- color codes , use rgbtohex(value) to get \127RRGGBB in hex for msg.
	ORANGE 		 = '170 170 0',
	NORMAL 		 = '255 255 255',	
	commodities  = '40 180 240',
	ship 			 = '126 255 128',
	lightweapon  = '127 127 127',
	heavyweapon  = '153 153 153',
	battery 		 = '255 185 53',
	ore			 = '100 150 127',
	mining		 = '100 150 0',
	turretweapon = '200 150 0',
	unknown		 = '255 255 255',
	seltextcolor = "0 0 128",
	--unseltextcolor = "255 255 255",
}

local function save_data()
	local id = (GetCharacterID())..'201'
	SaveSystemNotes(spickle({goods=TA.goods,stations=TA.stations}),id)
end

local function create_matrix(columndefs, defsort)
	local matrix
	local sortedlist
	local update_matrix
	local sort_key = defsort -- default to sort by distance
	local numcolumns = (#columndefs)
	local function set_sort_mode(mode)
		-- clicked on title of column
		sort_key = mode
		-- color the text accordingly
		for i=1,numcolumns do
			matrix:setattribute("BGCOLOR", 0, i, mode == i and tabseltextcolor or tabunseltextcolor)
		end
	end

	matrix = iup.pdasubsubmatrix{
		numcol = numcolumns,
		expand = "YES",
		size="200x100",
	}
	for i=1,numcolumns do
		matrix["ALIGNMENT"..i] = columndefs[i].alignment
		matrix["0:"..i] = columndefs[i].title
	end
	set_sort_mode(defsort)
	function matrix:bgcolor_cb(row, col)
		local c = bg_numbers[math.fmod(row,2)]
		return c[1],c[2],c[3],c[4],iup.DEFAULT
	end
	--matrix.fgcolor_cb = matrix.bgcolor_cb
	function matrix:click_cb(row, col)
		if row == 0 and tonumber(matrix.NUMLIN) > 0 then
			set_sort_mode(col)
			update_matrix(self)
		elseif columndefs.on_sel and sortedlist then
			columndefs.on_sel(matrix, row, sortedlist[row])
			update_matrix(self)
		end
	end
	function matrix:edition_cb()
		return iup.IGNORE
	end

	local function sort_list(sort_key, list)
		table.sort(list, columndefs[sort_key].fn)
	end

	local function reload_matrix(matrix, list)
		matrix.numlin = (#list)
	
		for index,v in ipairs(list) do
			columndefs.update_entry(matrix, index, v)
		end
	end
	
	local function reset_matrix(self, characterlist)
		matrix.dellin = "1--1"  -- one way of deleting all items in the matrix
		sortedlist = characterlist
		sort_list(sort_key, sortedlist)
		reload_matrix(matrix, sortedlist)
	end

	update_matrix = function(self)
		set_sort_mode(sort_key)
			sort_list(sort_key, sortedlist)
			reload_matrix(matrix, sortedlist)
	end

	matrix.update = update_matrix
	matrix.reset = reset_matrix

	return matrix
end


function TA.ui.Hide()
	HideDialog(TA.ui.MAIN,iup.CENTER,iup.CENTER)
end

-- iup stuff for the goods tab, 
TA.ui.Close = iup.stationbutton{title='Close', expand="HORIZONTAL", action = TA.ui.Hide }
TA.ui.goods_cu = iup.label{title='',expand="HORIZONTAL" }
TA.ui.goods_mass = iup.label{title='',expand="HORIZONTAL"}
TA.ui.goods_list = iup.stationsublist{dropdown="YES",expand="HORIZONTAL", action = function(text,num,value)
		if value ~= 0 then
			-- num is the [index] value 
			num = filter_colorcodes(num)
			TA.ui.goods_cu.title = comma_value(TA.goods[num].vol)
			TA.ui.goods_mass.title = comma_value(TA.goods[num].mass)

			-- fill the matrix with the relevant data.
			TA.ui.goods_tbl = {}
			for k,v in pairs(TA.goods[num]) do
				if type(v) == 'table' then
					table.insert(TA.ui.goods_tbl,1,{
							name = ShortLocationStr(TA.stations[k].sectorid)..' ['..GetStationName(k)..']',
							sell = v.sell or ' ',
							buy = v.buy or ' ',
						}
					)
				end
			end
			TA.ui.goods_matrix:reset(TA.ui.goods_tbl)
		end
	end
}

local goods_sorttbl = {
	[1] = {
		title = "Name",
		alignment = "ALEFT",
		fn = function(a,b) return a.name < b.name end,
		},
	[2] = {
		title = "Buy",
		alignment = "ARIGHT",
		fn = function(a,b)
			if a.buy == b.buy then return a.name < b.name
			else return gkmisc.strnatcmp(a.buy,b.buy) > 0
			end
		end,
		},
	[3] = {
		title = "Sell",
		alignment = "ARIGHT",
		fn = function(a,b)
			if a.sell == b.sell then return a.name < b.name
			else return gkmisc.strnatcmp(a.sell,b.sell) > 0
			end
		end,
		},
	update_entry = function(matrix, index, data)
		matrix:setcell(index, 1," "..data.name)
		matrix:setcell(index, 2," "..comma_value(data.buy))
		matrix:setcell(index, 3," "..comma_value(data.sell))
	end,
	on_sel = function (matrix, row, data)
	end,
	
}
TA.ui.goods_matrix = create_matrix(goods_sorttbl, 1)
local item_tab = iup.vbox{
	iup.hbox{
		iup.fill{size=5},
		iup.label{title='Item Name: ',expand="NO"},TA.ui.goods_list, iup.fill{size=5},
		iup.label{title='Volume: '},TA.ui.goods_cu,iup.fill{size=5},
		iup.label{title='Mass: '},TA.ui.goods_mass,
		iup.fill{size=5},
	},
	TA.ui.goods_matrix,
	tabtitle = 'Trade Goods',
}
--- end goods tab


-- iup stuff for the stations tab
TA.ui.station_buy  = iup.stationsublist{dropdown="YES", expand="HORIZONTAL", action = function(text,num,value)
--		if value ~= 0 then
--			-- num is the [index] value 
--		end
	end
}
TA.ui.station_sell  = iup.stationsublist{dropdown="YES",expand="HORIZONTAL", action = function(text,num,value)
--		if value ~= 0 then
--			-- num is the [index] value 
--		end
	end
}
local station_sorttbl = {
	[1] = {
		title = "Item Name",
		alignment = "ALEFT",
		fn = function(a,b) return a.name < b.name end,
		},
	[2] = {
		title = "Buy Price",
		alignment = "ARIGHT",
		fn = function(a,b)
			if a.buy == b.buy then return a.name < b.name
			else return gkmisc.strnatcmp(a.buy,b.buy) < 1
			--return a.buy < b.buy
			end
		end,
		},
	[3] = {
		title = "Sell Price",
		alignment = "ARIGHT",
		fn = function(a,b)
			if a.sell == b.sell then return a.name < b.name
			else return gkmisc.strnatcmp(a.sell,b.sell) > 0
			--return a.sell < b.sell
			end
		end,
		},
	[4] = {
		title = "Profit / CU",
		alignment = "ARIGHT",
		fn = function(a,b)
			if a.profit_cu == b.profit_cu then return a.name < b.name
			else return gkmisc.strnatcmp(a.profit_cu, b.profit_cu) > 0
			--return a.profit_cu > b.profit_cu
			end
		end,
		},
	[5] = {
		title = "Profit / Kg",
		alignment = "ARIGHT",
		fn = function(a,b)
			if a.profit_kg == b.profit_kg then return a.name < b.name
			else return gkmisc.strnatcmp(a.profit_kg,b.profit_kg) > 0
			--return a.profit_kg > b.profit_kg
			end
		end,
		},
	update_entry = function(matrix, index, data)
		matrix:setcell(index, 1," "..data.name)
		matrix:setcell(index, 2," "..comma_value(data.buy))
		matrix:setcell(index, 3," "..comma_value(data.sell))
		matrix:setcell(index, 4," "..comma_value(tonumber(string.format("%.2f",data.profit_cu or 0))))
		matrix:setcell(index, 5," "..comma_value(tonumber(string.format("%.2f",data.profit_kg or 0))))
	end,
	on_sel = function(matrix, row, data)
	end,
}
	
TA.ui.station_matrix =  create_matrix(station_sorttbl, 4)
TA.ui.station_go = iup.stationbutton{title='Go', expand="NO", action = function()
		TA.ui.stations_tbl = {}
		local buy_id = sorted.stations[tonumber(TA.ui.station_buy.value)][2]
		local sell_id = sorted.stations[tonumber(TA.ui.station_sell.value)][2]
		
		for k,v in pairs(TA.goods) do
			if (v[buy_id] and v[buy_id].sell) and 
			(v[sell_id] and v[sell_id].buy) and v[sell_id].buy > v[buy_id].sell then
				local cu = tonumber(v.vol) or 1
				local mass = tonumber(v.mass) or tonumber(v.mass:match("(%d+) kg")) or 1
				local sell = v[sell_id].buy
				local buy = v[buy_id].sell
				table.insert(TA.ui.stations_tbl, 1, {
					name = k,
					sell = sell,
					buy = buy,
					profit_cu = tonumber(sell - buy)/cu or tonumber(sell - buy)..'p',
					profit_kg = tonumber(sell - buy)/mass or '-na-',
					}
				)
			end
		end
		if #TA.ui.stations_tbl == 0 then
			table.insert(TA.ui.stations_tbl, 1, {
				name = 'No',
				buy = 'Results',
				sell = 'Found',
			})
		end
		TA.ui.station_matrix:reset(TA.ui.stations_tbl)
	end
}

local station_tab = iup.vbox{
	iup.hbox{
		iup.fill{size=5},
		iup.label{title='Buy: ',expand="NO"},TA.ui.station_buy, iup.fill{size=5},
		iup.label{title='Sell: ',expand="NO"},TA.ui.station_sell, iup.fill{size=5},
		TA.ui.station_go,
		iup.fill{size=5},
	},
	TA.ui.station_matrix,
	tabtitle = 'Stations',
}
--- end station tab


local function reset_data()
	local tbl = {}

	--this is for the list box
	local count = 1
	for k,v in pairs(TA.goods) do
		if k:match(" Ore") then
			table.insert(tbl, rgbtohex(TA.ui.color['ore'])..k)
		elseif k:match("Mining") or k:match("Mineral Scanner") then
			table.insert(tbl, rgbtohex(TA.ui.color['mining'])..k)
		elseif not TA.ui.color[v.type] then
			table.insert(tbl, rgbtohex(TA.ui.color.unknown)..k)
		else
			table.insert(tbl, rgbtohex(TA.ui.color[v.type])..k)
		end
	end
	table.sort(tbl, function(a,b) return gkmisc.strnatcasecmp(a,b) < 1 end)
	TA.ui.goods_list[count] = nil
	TA.ui.station_matrix:reset({})
	TA.ui.goods_matrix:reset({})
	
	for k,v in pairs(tbl) do
		TA.ui.goods_list[count] = v
		count = count + 1
	end
	
	-- stations
	count = 1
	sorted.stations = {}
	for k,v in pairs(TA.stations) do
		local location = ShortLocationStr(TA.stations[k].sectorid)..' ['..GetStationName(k)..']'
		table.insert(sorted.stations,{location, k})
	end
	table.sort(sorted.stations, function(a,b) return gkmisc.strnatcasecmp(a[1],b[1]) < 1 end)
	TA.ui.station_buy[count] = nil
	TA.ui.station_sell[count] = nil
	
	for k,v in pairs(sorted.stations) do
		TA.ui.station_buy[count] = v[1]
		TA.ui.station_sell[count] = v[1]
		count = count + 1
	end

	-- width of the matrix columns
	local width = getwidth(TA.ui.goods_matrix) - Font.Default
	TA.ui.goods_matrix.width1 = width/3 * 2
	TA.ui.goods_matrix.width2 = width/6
	TA.ui.goods_matrix.width3 = width/6

	TA.ui.station_matrix.width1 = width / 2  -- item name
	TA.ui.station_matrix.width2 = width / 8 -- buy price
	TA.ui.station_matrix.width3 = width / 8 -- sell price
	TA.ui.station_matrix.width4 = width / 8 -- profit / cu
	TA.ui.station_matrix.width5 = width / 8 -- profit / kg
end

local function load_data()
	local id = GetCharacterID()
	if id then 
		local load = unspickle(LoadSystemNotes(id..'201'))
		purchaseprint ('loading TA data for '..GetPlayerName())
		TA.goods = load.goods or {}
		TA.stations = load.stations or {}
		startup = false
	end
end

function TA.ui.Show()
	reset_data()
	ShowDialog(TA.ui.MAIN,iup.CENTER,iup.CENTER)
end


-- /command function
function TA.commands (data,args)
  if args then
    if args[1]:lower() == 'version' then
      purchaseprint(rgbtohex(TA.ui.color.NORMAL)..'TA version: '..rgbtohex(TA.ui.color.ORANGE)..TA.version..rgbtohex(TA.ui.color.NORMAL)..' By Mad Miner Moda')
	elseif args[1]:lower() == 'purge' then
		TA.goods = {}
		TA.stations = {}
		save_data()
		reset_data()
		purchaseprint("All TA data purged")
    end
  else -- no args specified, launch GUI
	TA.ui.Show()
  end -- args check
end

TA.ui.MAIN = iup.dialog{
	iup.stationhighopacityframe{
		iup.stationhighopacityframebg{
			iup.vbox{
				iup.pda_root_tabs{
					item_tab,
					station_tab,
				},
				iup.hbox{TA.ui.Close},
			},
		},
	},
	border="YES",
	topmost="YES",
	size='%90x%90',
	DEFAULTESC=TA.ui.Close,
	BGCOLOR = "255 0 0 0 *",
	title = TA.version,
}
TA.ui.MAIN:map()


-- most all event handlers goes here.
function TA.ui.MAIN:OnEvent(eventname,...)
	if startup then
		load_data()
	elseif eventname == 'SYSTEM_BEST_PRICE' then
		
		local arg = { ... }
		local localItemID = arg[1]
		local name = GetInventoryItemName(localItemID)
		local quant = GetInventoryItemQuantity(localItemID)
		local buy = arg[2]/quant
		local stat = string.sub(arg[3],1,-3) -- remove the 01
		if TA.goods[name] then -- can only update if we have seen the item
			-- weird glitch ... need to translate back and forth to get real ID
			-- and also each system seems to like different LocationStr 
			local sectorID = SectorIDFromLocationStr(AbbrLocationStr(stat)) or
				SectorIDFromLocationStr(ShortLocationStr(stat)) or
				SectorIDFromLocationStr(LocationStr(stat))
				if sectorID then -- added buffer in case above fails to return
					for k,v in pairs(TA.stations) do -- must have been to that station
						if v.sectorid == sectorID then -- translate sector to station
							local stationid = k
							TA.goods[name][stationid] = TA.goods[name][stationid] or {}
								TA.goods[name][stationid].buy = buy
					end -- did we visit station yet?	
				end -- for stationid search	
			end -- if sectorId nil
		end -- if name exists
		save_data()

	elseif eventname == 'PLAYER_LOGGED_OUT' then
		startup = true
	end
	
	if eventname == 'ENTERED_STATION' and not startup then
		if GetCurrentStationType() == 0 and PlayerInStation() then -- exclude capships
			-- data collection here.
			local stationid = GetStationLocation()
			
			if not TA.stations[stationid] then
				local longname = GetStationName()
				purchaseprint ('Creating TA station data for '..longname)
				if not TA.stations then TA.stations = {} end
				TA.stations[stationid] = {}
				TA.stations[stationid].sectorid = GetCurrentSectorid()
				TA.stations[stationid].systemid = GetCurrentSystemid()
				TA.stations[stationid].name = longname
			end
			
			-- all items for sale at station
			for x = 1, GetNumStationMerch() do 
				local name = GetStationMerchInfo(x).name
				local mass = GetStationMerchInfo(x).mass or GetStationMerchInfo(x).Mass
				mass = math.floor(1000*(mass or 0)+0.5)
				local vol = GetStationMerchInfo(x).volume
				local price = GetStationMerchInfo(x).price
				local type = GetStationMerchInfo(x).type
				if type then
					if not TA.goods then TA.goods = {} end
					TA.goods[name] = TA.goods[name] or {}
					TA.goods[name][stationid] = TA.goods[name][stationid] or {}
					TA.goods[name][stationid].sell = price
					TA.goods[name].mass = TA.goods[name].mass or mass
					TA.goods[name].vol = TA.goods[name].vol or vol
					TA.goods[name].type = TA.goods[name].type or type
				end
			end
			
			-- stuff you have for sale
			for k,v in StationSellableInventoryPairs() do
				local name = v.name
				local mass = v.mass ~= '' and v.mass or v.longdesc:match("Mass: (%d+%skg)\n") or '??'
				local volume = v.Volume ~= '' and v.Volume or v.longdesc:match("Volume %(as cargo%): (%d+)") or '??'
				if tonumber(volume) then volume = volume..' cu' end				
				if v.type then
					TA.goods[name] = TA.goods[name] or {}
					TA.goods[name][stationid] = TA.goods[name][stationid] or {}
					TA.goods[name][stationid].buy = v.price
					TA.goods[name].mass = TA.goods[name].mass or mass
					TA.goods[name].vol = TA.goods[name].vol or volume
					TA.goods[name].type = TA.goods[name].type or v.type
				end
			end
			save_data() 
			purchaseprint('TA saving data')
		end	-- capship check
	end -- if eventname 
end -- TA.ui.MAIN:OnEvent

 
-- this cute snipet allows the app to close/hide when the bound key is pressed
-- i.e. bind 'x' to /storm press x once it launches /storm, press again it closes
function TA.ui.MAIN:k_any(char)
	local command = gkinterface.GetCommandForKeyboardBind(char)
	if command == 'ta' then
		TA.ui.Hide()
		return iup.IGNORE
	end
	return iup.CONTINUE
end -- TA.ui.MAIN:k_any()


---- registration ----
RegisterUserCommand("ta",TA.commands)
--RegisterEvent(TA.ui.MAIN, 'SYSTEM_BEST_PRICE')
RegisterEvent(TA.ui.MAIN, 'ENTERED_STATION') -- when a player docks
RegisterEvent(TA.ui.MAIN, 'PLAYER_LOGGED_OUT')
RegisterEvent(TA.ui.MAIN, 'PLAYER_ENTERED_GAME') -- starting the game
